﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;
using DlhSoft.Windows.Data;

namespace GanttChartTaskListViewDependencyPropertiesSample
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        public Window1()
        {
            InitializeComponent();
        }

        private void EditMenuItem_Click(object sender, RoutedEventArgs e)
        {
            MenuItem editMenuItem = sender as MenuItem;
            // Determine the predecessor object that the edit menu item refers to.
            Predecessor predecessor = editMenuItem.DataContext as Predecessor;
            // Open the Dependency Properties dialog allowing the end user to edit the properties of the existing dependency.
            DependencyPropertiesDialog dependencyPropertiesDialog = new DependencyPropertiesDialog { Owner = this, DataContext = predecessor };
            dependencyPropertiesDialog.ShowDialog();
        }

        // Stores the newly created dependency event arguments retreived in the NewPredecessor event handler, until the current drag and drop operation is completed.
        private NewItemEventArgs newlyCreatedDependencyEventArguments;

        private void GanttChartTaskListView_NewPredecessor(object sender, NewItemEventArgs e)
        {
            newlyCreatedDependencyEventArguments = e;
        }

        private void GanttChartTaskListView_PreviewMouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            newlyCreatedDependencyEventArguments = null;
        }

        // Stores a timer object that postpones showing of the Dependency Properties dialog until the current drag and drop operation is completed when a new dependency is created.
        private DispatcherTimer newlyCreatedPredecessorTimer;

        private void GanttChartTaskListView_PreviewMouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            if (newlyCreatedPredecessorTimer == null)
            {
                newlyCreatedPredecessorTimer = new DispatcherTimer { Interval = TimeSpan.Zero };
                newlyCreatedPredecessorTimer.Tick += NewlyCreatedPredecessorTimer_Tick;
            }
            newlyCreatedPredecessorTimer.Start();
        }

        private void NewlyCreatedPredecessorTimer_Tick(object sender, EventArgs e)
        {
            newlyCreatedPredecessorTimer.Stop();

            if (newlyCreatedDependencyEventArguments == null)
                return;

            // Determine the newly created predecessor object.
            Predecessor predecessor = GetNewlyCreatedPredecessor();
            if (predecessor != null)
            {
                // Open the Dependency Properties dialog allowing the end user to edit the properties of the newly created dependency.
                DependencyPropertiesDialog dependencyPropertiesDialog = new DependencyPropertiesDialog { Owner = this, DataContext = predecessor };
                if (dependencyPropertiesDialog.ShowDialog() != true)
                {
                    // When the dialog is cancelled, delete the newly created dependency.
                    Task parentTask = predecessor.ParentTask;
                    object predecessorItem = predecessor.Item;
                    parentTask.Predecessors.RemoveItem(predecessorItem);
                }
            }

            newlyCreatedDependencyEventArguments = null;
        }

        // Determines the newly created predecessor object based on the newly created dependency event arguments retreived during the previous NewPredecessor event handler call.
        private Predecessor GetNewlyCreatedPredecessor()
        {
            Task parentTask = newlyCreatedDependencyEventArguments.ParentEntity as Task;
            object taskItem = newlyCreatedDependencyEventArguments.Parameters[0];
            foreach (Predecessor predecessor in parentTask.Predecessors)
            {
                if (predecessor.Task.Item == taskItem)
                    return predecessor;
            }
            return null;
        }
    }
}
